@@ -43,6 +43,7 @@
 #include "alloc.h"
 #include "api.h"
 #include "events.h"
+#include "openapi.h"
 #include "parsers.h"
 #include "parsing.h"
 #include "slurmdb_helpers.h"
@@ -50,6 +51,7 @@
 #define MAGIC_FOREACH_LIST_FLAG 0xa1d4acd2
 #define MAGIC_FOREACH_LIST 0xaefa2af3
 #define MAGIC_FOREACH_NT_ARRAY 0xaba1be2b
+#define MAGIC_FOREACH_PARSE_MARRAY 0xa081be2b
 
 typedef struct {
 	int magic;
@@ -82,6 +84,14 @@ typedef struct {
 	data_t *parent_path;
 } foreach_nt_array_t;
 
+typedef struct {
+	int magic; /* MAGIC_FOREACH_PARSE_MARRAY */
+	args_t *args;
+	const parser_t *const array;
+	data_t *parent_path;
+	data_t *path;
+} parse_marray_args_t;
+
 static void _set_flag_bit(const parser_t *const parser, void *dst,
 			  const flag_bit_t *bit, bool matched, const char *path,
 			  data_t *src)
@@ -168,8 +178,11 @@ static char *_flag_parent_path(char **path_ptr,
 	if (*path_ptr)
 		return *path_ptr;
 
+	if (is_fast_mode(args->args))
+		return NULL;
+
 	ppath = clone_source_path_index(args->parent_path, args->index);
-	set_source_path(path_ptr, ppath);
+	set_source_path(path_ptr, args->args, ppath);
 	FREE_NULL_DATA(ppath);
 
 	return *path_ptr;
@@ -231,13 +244,12 @@ static int _parse_flag(void *dst, const parser_t *const parser, data_t *src,
 {
 	int rc = SLURM_SUCCESS;
 	char *path = NULL;
-	data_t *ppath = data_copy(NULL, parent_path);
 	foreach_flag_parser_args_t fargs = {
 		.magic = MAGIC_FOREACH_LIST_FLAG,
 		.args = args,
 		.parser = parser,
 		.dst = dst,
-		.parent_path = ppath,
+		.parent_path = parent_path,
 	};
 
 	xassert(args->magic == MAGIC_ARGS);
@@ -250,7 +262,8 @@ static int _parse_flag(void *dst, const parser_t *const parser, data_t *src,
 		if (_foreach_flag_parser(src, &fargs) != DATA_FOR_EACH_CONT) {
 			rc = on_error(PARSING, parser->type, args,
 				      ESLURM_DATA_FLAGS_INVALID,
-				      set_source_path(&path, ppath), __func__,
+				      set_source_path(&path, args, parent_path),
+				      __func__,
 				      "Parsing single flag \"%s\" failed",
 				      data_get_string(src));
 
@@ -259,8 +272,8 @@ static int _parse_flag(void *dst, const parser_t *const parser, data_t *src,
 	} else if (data_get_type(src) != DATA_TYPE_LIST) {
 		rc = on_error(PARSING, parser->type, args,
 			      ESLURM_DATA_FLAGS_INVALID_TYPE,
-			      set_source_path(&path, ppath), __func__,
-			      "Expected a List but found a %s",
+			      set_source_path(&path, args, parent_path),
+			      __func__, "Expected a List but found a %s",
 			      data_get_type_string(src));
 
 		goto cleanup;
@@ -272,15 +285,13 @@ static int _parse_flag(void *dst, const parser_t *const parser, data_t *src,
 	 */
 	} else if (data_list_for_each(src, _foreach_flag_parser, &fargs) < 0) {
 		rc = on_error(PARSING, parser->type, args,
-			      ESLURM_DATA_FLAGS_INVALID, set_source_path(&path,
-									 ppath),
-			      __func__,
-			      "Parsing flags failed");
+			      ESLURM_DATA_FLAGS_INVALID,
+			      set_source_path(&path, args, parent_path),
+			      __func__, "Parsing flags failed");
 		goto cleanup;
 	}
 
 cleanup:
-	FREE_NULL_DATA(ppath);
 	xfree(path);
 	return rc;
 }
@@ -291,30 +302,30 @@ static data_for_each_cmd_t _foreach_parse_list(data_t *src, void *arg)
 	foreach_list_t *args = arg;
 	const parser_t *const parser = args->parser;
 	const parser_t *const lparser = find_parser_by_type(parser->list_type);
-	void *obj = alloc_parser_obj(lparser);
-	data_t *ppath = data_copy(NULL, args->parent_path);
-	data_t *ppath_last = data_get_list_last(ppath);
+	void *obj = NULL;
+	data_t *ppath = NULL;
 
 	xassert(args->magic == MAGIC_FOREACH_LIST);
 	check_parser(parser);
 	check_parser(lparser);
 	xassert(!args->dlist); /* only for dumping */
 	xassert((args->index > 0) || (args->index == -1));
-	xassert((lparser->size == NO_VAL) || (xsize(obj) == lparser->size));
 
 	if (args->index < 0)
 		args->index = 0;
 
-	/* Use jq style array zero based array notation */
-	data_set_string_fmt(ppath_last, "%s[%zu]",
-			    data_get_string(ppath_last),
-			    args->index);
+	if (!is_fast_mode(args->args)) {
+		data_t *ppath_last;
 
-	if ((rc = parse(obj, NO_VAL, lparser, src, args->args, ppath))) {
-		log_flag(DATA, "%s object at 0x%"PRIxPTR" freed due to parser error: %s",
-			 lparser->obj_type_string, (uintptr_t) obj,
-			 slurm_strerror(rc));
-		free_parser_obj(lparser, obj);
+		ppath = data_copy(NULL, args->parent_path);
+		ppath_last = data_get_list_last(ppath);
+
+		/* Use jq style array zero based array notation */
+		data_set_string_fmt(ppath_last, "%s[%zu]",
+				    data_get_string(ppath_last), args->index);
+	}
+
+	if ((rc = parse(&obj, NO_VAL, lparser, src, args->args, ppath))) {
 		FREE_NULL_DATA(ppath);
 		return DATA_FOR_EACH_FAIL;
 	}
@@ -347,30 +358,31 @@ static int _parse_list(const parser_t *const parser, void *dst, data_t *src,
 	check_parser(parser);
 
 	log_flag(DATA, "%s: BEGIN: list parsing %s{%s(0x%"PRIxPTR")} to List 0x%"PRIxPTR" via parser %s(0x%"PRIxPTR")",
-		__func__, set_source_path(&path, parent_path),
+		__func__, set_source_path(&path, args, parent_path),
 		data_get_type_string(src), (uintptr_t) src,
 		(uintptr_t) dst, parser->type_string, (uintptr_t) parser
 	);
 
 	if (!list_args.list)
-		list_args.list = list_create(parser_obj_free_func(parser));
+		list_args.list = list_create((ListDelF) parser->free);
 
 	xassert(list_count(list_args.list) >= 0);
 
-	if (data_get_type(src) != DATA_TYPE_LIST) {
+	if (data_get_type(src) == DATA_TYPE_LIST) {
+		if (data_list_for_each(src, _foreach_parse_list,
+				       &list_args) < 0)
+			rc = ESLURM_REST_FAIL_PARSING;
+	} else if (data_convert_type(src, DATA_TYPE_STRING) ==
+		   DATA_TYPE_STRING) {
+		/* Assume List is just a single entry */
+		if (_foreach_parse_list(src, &list_args) != DATA_FOR_EACH_CONT)
+			rc = ESLURM_REST_FAIL_PARSING;
+	} else {
 		rc = on_error(PARSING, parser->type, args,
-			      ESLURM_DATA_FLAGS_INVALID_TYPE, path, __func__,
-			      "Expected List but found a %s",
+			      ESLURM_DATA_EXPECTED_LIST,
+			      set_source_path(&path, args, parent_path),
+			      __func__, "Expected List but found a %s",
 			      data_get_type_string(src));
-		goto cleanup;
-	}
-
-	if (data_list_for_each(src, _foreach_parse_list, &list_args) < 0) {
-		rc = on_error(PARSING, parser->type, args,
-			ESLURM_REST_FAIL_PARSING,
-			set_source_path(&path, parent_path),
-			__func__, "parsing failed");
-		goto cleanup;
 	}
 
 	if (!rc) {
@@ -378,13 +390,14 @@ static int _parse_list(const parser_t *const parser, void *dst, data_t *src,
 		list_args.list = NULL;
 	}
 
-cleanup:
 	log_flag(DATA, "%s: END: list parsing %s{%s(0x%"PRIxPTR")} to List 0x%"PRIxPTR" via parser %s(0x%"PRIxPTR") rc[%d]:%s",
 		__func__, path, data_get_type_string(src),
 		(uintptr_t) src, (uintptr_t) dst, parser->type_string,
 		(uintptr_t) parser, rc, slurm_strerror(rc)
 	);
 
+	if (rc)
+		*list = NULL;
 	FREE_NULL_LIST(list_args.list);
 	xfree(path);
 	return rc;
@@ -404,7 +417,7 @@ static int _parse_pointer(const parser_t *const parser, void *dst, data_t *src,
 			     !data_get_list_length(src);
 
 	xassert(!*ptr);
-	*ptr = alloc_parser_obj(pt);
+	*ptr = alloc_parser_obj(parser);
 
 	if (is_empty_dict || is_empty_list) {
 		/*
@@ -416,8 +429,13 @@ static int _parse_pointer(const parser_t *const parser, void *dst, data_t *src,
 		return SLURM_SUCCESS;
 	}
 
-	if ((rc = parse(*ptr, NO_VAL, pt, src, args, parent_path)))
-		free_parser_obj(pt, *ptr);
+	if ((rc = parse(*ptr, NO_VAL, pt, src, args, parent_path))) {
+		log_flag(DATA, "%s object at 0x%"PRIxPTR" freed due to parser error: %s",
+			 pt->obj_type_string, (uintptr_t) ptr,
+			 slurm_strerror(rc));
+		free_parser_obj(parser, *ptr);
+		*ptr = NULL;
+	}
 
 	return rc;
 }
@@ -427,8 +445,7 @@ static data_for_each_cmd_t _foreach_array_entry(data_t *src, void *arg)
 	int rc;
 	foreach_nt_array_t *args = arg;
 	void *obj = NULL;
-	data_t *ppath = data_copy(NULL, args->parent_path);
-	data_t *ppath_last = data_get_list_last(ppath);
+	data_t *ppath = NULL;
 
 	xassert(args->magic == MAGIC_FOREACH_NT_ARRAY);
 	xassert((args->index > 0) || (args->index == -1));
@@ -436,10 +453,16 @@ static data_for_each_cmd_t _foreach_array_entry(data_t *src, void *arg)
 	if (args->index < 0)
 		args->index = 0;
 
-	/* Use jq style array zero based array notation */
-	data_set_string_fmt(ppath_last, "%s[%d]",
-			    data_get_string(ppath_last),
-			    args->index);
+	if (!is_fast_mode(args->args)) {
+		data_t *ppath_last;
+
+		ppath = data_copy(NULL, args->parent_path);
+		ppath_last = data_get_list_last(ppath);
+
+		/* Use jq style array zero based array notation */
+		data_set_string_fmt(ppath_last, "%s[%d]",
+				    data_get_string(ppath_last), args->index);
+	}
 
 	if (args->parser->model == PARSER_MODEL_NT_PTR_ARRAY)
 		obj = alloc_parser_obj(args->parser);
@@ -482,11 +505,12 @@ static int _parse_nt_array(const parser_t *const parser, void *dst, data_t *src,
 
 	xassert(args->magic == MAGIC_ARGS);
 
-	if (data_get_type(src) != DATA_TYPE_LIST) {
+	if ((data_get_type(src) != DATA_TYPE_LIST) &&
+	    (data_convert_type(src, DATA_TYPE_STRING) != DATA_TYPE_STRING)) {
 		rc = on_error(PARSING, parser->type, args,
-			      ESLURM_DATA_FLAGS_INVALID_TYPE,
-			      set_source_path(&path, parent_path), __func__,
-			      "Expected List but found a %s",
+			      ESLURM_DATA_EXPECTED_LIST,
+			      set_source_path(&path, args, parent_path),
+			      __func__, "Expected List but found a %s",
 			      data_get_type_string(src));
 		goto cleanup;
 	}
@@ -499,8 +523,14 @@ static int _parse_nt_array(const parser_t *const parser, void *dst, data_t *src,
 		fargs.sarray = xcalloc(data_get_list_length(src) + 1,
 				       sizeof(fargs.parser->size));
 
-	if (data_list_for_each(src, _foreach_array_entry, &fargs) < 0)
-		goto cleanup;
+	if (data_get_type(src) == DATA_TYPE_LIST) {
+		if (data_list_for_each(src, _foreach_array_entry, &fargs) < 0)
+			goto cleanup;
+	} else if (data_get_type(src) == DATA_TYPE_STRING) {
+		/* Assume List is just a single entry */
+		if (_foreach_array_entry(src, &fargs) != DATA_FOR_EACH_CONT)
+			rc = ESLURM_REST_FAIL_PARSING;
+	}
 
 	if (parser->model == PARSER_MODEL_NT_PTR_ARRAY) {
 		void ***array_ptr = dst;
@@ -525,27 +555,220 @@ cleanup:
 	return rc;
 }
 
+static void _parser_linked_flag(args_t *args, const parser_t *const array,
+				const parser_t *const parser, data_t *src,
+				void *dst, data_t *parent_path,
+				const flag_bit_t *bit, uint64_t *set)
+{
+	char *path = NULL;
+	data_t *ppath = NULL;
+	bool matched;
+
+	src = data_resolve_dict_path(src, bit->name);
+
+	if (!is_fast_mode(args)) {
+		ppath = data_copy(NULL, parent_path);
+		openapi_append_rel_path(ppath, bit->name);
+		set_source_path(&path, args, ppath);
+	}
+
+	if (!src) {
+		matched = false;
+	} else if (data_convert_type(src, DATA_TYPE_BOOL) != DATA_TYPE_BOOL) {
+		matched = false;
+
+		on_warn(PARSING, parser->type, args, path, __func__,
+			"Unable to convert to boolean from %s. Flag %s is being treated as false.",
+			data_get_type_string(src), bit->name);
+
+	} else {
+		matched = data_get_bool(src);
+	}
+
+	if (bit->type == FLAG_BIT_TYPE_BIT)
+		_set_flag_bit(parser, dst, bit, matched, path, src);
+	else if (bit->type == FLAG_BIT_TYPE_EQUAL) {
+		if (matched || ((~(*set) & bit->mask) == bit->mask))
+			_set_flag_bit_equal(parser, dst, bit, matched, path,
+					    src);
+		*set |= bit->mask;
+	} else {
+		fatal_abort("%s: invalid bit_flag_t", __func__);
+	}
+
+	log_flag(DATA, "%s: parsed flag %s{%s(0x%" PRIxPTR ")} to %s(0x%" PRIxPTR "+%zd)->%s & 0x%"PRIx64" & %s=0x%"PRIx64" via array parser %s(0x%" PRIxPTR ")=%s(0x%" PRIxPTR ")",
+		 __func__, path, data_get_type_string(src),
+		 (uintptr_t) src, array->obj_type_string, (uintptr_t) dst,
+		 parser->ptr_offset, parser->field_name, bit->mask,
+		 bit->flag_name, bit->value, parser->obj_type_string,
+		 (uintptr_t) parser, array->type_string, (uintptr_t) array);
+
+	xfree(path);
+	FREE_NULL_DATA(ppath);
+}
+
+static data_for_each_cmd_t _foreach_parse_marray(const char *key, data_t *data,
+						 void *arg)
+{
+	parse_marray_args_t *aargs = arg;
+	parse_marray_args_t cargs = *aargs;
+	args_t *args = aargs->args;
+	const parser_t *const array = aargs->array;
+	char *path = NULL;
+
+	xassert(aargs->magic == MAGIC_FOREACH_PARSE_MARRAY);
+	xassert(cargs.magic == MAGIC_FOREACH_PARSE_MARRAY);
+	xassert(array->model == PARSER_MODEL_ARRAY);
+	xassert(!is_fast_mode(args));
+
+	cargs.parent_path = data_copy(NULL, aargs->parent_path);
+	openapi_append_rel_path(cargs.parent_path, key);
+
+	cargs.path = data_copy(NULL, aargs->path);
+	data_set_string(data_list_append(cargs.path), key);
+
+	for (int i = 0; i < array->field_count; i++) {
+		bool match;
+		const parser_t *const parser = &array->fields[i];
+		data_t *fpath;
+
+		if (parser->model == PARSER_MODEL_ARRAY_SKIP_FIELD)
+			continue;
+
+		if (parser->model ==
+		    PARSER_MODEL_ARRAY_LINKED_EXPLODED_FLAG_ARRAY_FIELD) {
+			const parser_t *const fp =
+				find_parser_by_type(parser->type);
+
+			for (int i = 0; i < fp->flag_bit_array_count; i++) {
+				const flag_bit_t *bit = &fp->flag_bit_array[i];
+
+				if (!xstrcasecmp(key, bit->name)) {
+					if (slurm_conf.debug_flags &
+					    DEBUG_FLAG_DATA) {
+						char *p = NULL;
+						data_list_join_str(&p,
+								   cargs.path,
+								   "/");
+
+						log_flag(DATA, "%s: matched %s as bitflag %s",
+						      __func__, p, bit->name);
+
+						xfree(p);
+					}
+					goto cleanup;
+				}
+			}
+		}
+
+		fpath = data_new();
+		(void) data_list_split_str(fpath, parser->key, "/");
+		match = data_check_match(fpath, cargs.path, false);
+		FREE_NULL_DATA(fpath);
+
+		if (match) {
+			if (slurm_conf.debug_flags & DEBUG_FLAG_DATA) {
+				char *p = NULL;
+				data_list_join_str(&p, cargs.path, "/");
+				log_flag(DATA, "%s: matched %s to %s",
+				      __func__, p, parser->key);
+				xfree(p);
+			}
+			goto cleanup;
+		}
+	}
+
+	if (data_get_type(data) == DATA_TYPE_DICT) {
+		/* still unknown, so try next level of tree */
+		(void) data_dict_for_each(data, _foreach_parse_marray, &cargs);
+		goto cleanup;
+	}
+
+	on_warn(PARSING, array->type, args,
+		set_source_path(&path, args, cargs.parent_path),
+		__func__, "Ignoring unknown field \"%s\" of type %s in %s", key,
+		data_get_type_string(data), array->type_string);
+
+cleanup:
+	FREE_NULL_DATA(cargs.path);
+	FREE_NULL_DATA(cargs.parent_path);
+	xfree(path);
+	return DATA_FOR_EACH_CONT;
+}
+
+/*
+ * Try to guess if the user actually set the value or if the value was dumped
+ * with the other oversubsribed values which means it doesn't need to be logged
+ */
+static int _is_duplicate_linked_parser_value(args_t *args,
+					     const parser_t *const array,
+					     const parser_t *const parser,
+					     data_t *src_obj, data_t *src,
+					     data_t *parent_path)
+{
+	if (parser->field_name_overloads == 1)
+		return false;
+
+	for (int i = 0; i < array->field_count; i++) {
+		if ((array->fields[i].field_name_overloads != 1) &&
+		    !xstrcmp(array->fields[i].field_name, parser->field_name) &&
+		    !data_check_match(src,
+				      data_key_get(src_obj,
+						   array->fields[i].key),
+				      false)) {
+			return false;
+		}
+	}
+
+	return true;
+}
+
 /* parser linked parser inside of parser array */
 static int _parser_linked(args_t *args, const parser_t *const array,
 			  const parser_t *const parser, data_t *src, void *dst,
 			  data_t *parent_path)
 {
-	int rc;
-	data_t *ppath = data_copy(NULL, parent_path);
+	int rc = SLURM_ERROR;
+	data_t *ppath = NULL, *src_obj = src;
 	char *path = NULL;
 
 	check_parser(parser);
 	verify_parser_sliced(parser);
 
+	if (parser->model ==
+	    PARSER_MODEL_ARRAY_LINKED_EXPLODED_FLAG_ARRAY_FIELD) {
+		const parser_t *const fp = find_parser_by_type(parser->type);
+		uint64_t set = 0;
+
+		rc = SLURM_SUCCESS;
+
+		if (parser->ptr_offset != NO_VAL)
+			dst += parser->ptr_offset;
+
+		for (int i = 0; i < fp->flag_bit_array_count; i++) {
+			const flag_bit_t *bit = &fp->flag_bit_array[i];
+
+			_parser_linked_flag(args, array, fp, src, dst,
+					    parent_path, bit, &set);
+		}
+
+		goto cleanup;
+	}
+
+	if (!is_fast_mode(args))
+		ppath = data_copy(NULL, parent_path);
+
 	/* only look for child via key if there was one defined */
 	if (parser->key) {
 		src = data_resolve_dict_path(src, parser->key);
-		openapi_append_rel_path(ppath, parser->key);
+
+		if (!is_fast_mode(args))
+			openapi_append_rel_path(ppath, parser->key);
 	}
 
 	if (parser->model == PARSER_MODEL_ARRAY_REMOVED_FIELD) {
 		log_flag(DATA, "%s: skip parsing removed %s object %s(0x%" PRIxPTR ") via parser %s(0x%" PRIxPTR ")",
-			__func__, set_source_path(&path, ppath),
+			__func__, set_source_path(&path, args, ppath),
 			parser->obj_type_string, (uintptr_t) dst,
 			parser->type_string, (uintptr_t) src);
 		rc = SLURM_SUCCESS;
@@ -556,8 +779,9 @@ static int _parser_linked(args_t *args, const parser_t *const array,
 		if (parser->required) {
 			if ((rc = on_error(PARSING, parser->type, args,
 					   ESLURM_DATA_PATH_NOT_FOUND,
-					   set_source_path(&path, ppath),
-					   __func__, "Missing required field '%s' in dictionary",
+					   set_source_path(&path, args, ppath),
+					   __func__,
+					   "Missing required field '%s' in dictionary",
 					   parser->key)))
 				goto cleanup;
 		} else {
@@ -597,6 +821,15 @@ static int _parser_linked(args_t *args, const parser_t *const array,
 
 	xassert(parser->model == PARSER_MODEL_ARRAY_LINKED_FIELD);
 
+	if (!is_fast_mode(args) && parser->deprecated &&
+	    (parser->deprecated <= SLURM_MIN_PROTOCOL_VERSION) &&
+	    !_is_duplicate_linked_parser_value(args, array, parser, src_obj,
+					       src, parent_path)) {
+		on_warn(PARSING, parser->type, args,
+			set_source_path(&path, args, ppath), __func__,
+			"Field \"%s\" is deprecated", parser->key);
+	}
+
 	log_flag(DATA, "%s: BEGIN: parsing %s{%s(0x%" PRIxPTR ")} to %s(0x%" PRIxPTR "+%zd)%s%s=%s(0x%" PRIxPTR ") via array parser %s(0x%" PRIxPTR ")=%s(0x%" PRIxPTR ")",
 		 __func__, path, data_get_type_string(src),
 		 (uintptr_t) src, array->obj_type_string, (uintptr_t) dst,
@@ -628,6 +861,7 @@ static void _parse_check_openapi(const parser_t *const parser, data_t *src,
 {
 	char *path = NULL;
 	const char *oas_type, *oas_format, *found_type, *found_format;
+	data_type_t oas_data_type;
 	openapi_type_format_t found;
 
 	if (data_get_type(src) == DATA_TYPE_NULL)
@@ -646,21 +880,22 @@ static void _parse_check_openapi(const parser_t *const parser, data_t *src,
 	found = openapi_data_type_to_type_format(data_get_type(src));
 	found_type = openapi_type_format_to_type_string(found);
 	found_format = openapi_type_format_to_format_string(found);
+	oas_data_type = openapi_type_format_to_data_type(parser->obj_openapi);
+
+	xassert(!is_complex_mode(args));
 
 	/*
 	 * Warn as this is user provided data and the parser may accept
 	 * the format anyway. Steer the user towards the formats given
 	 * in the OpenAPI specification as set in the parser.
 	 */
-	on_warn(PARSING, parser->type, args, set_source_path(&path,
-							     parent_path),
-		__func__,
+	on_warn(PARSING, parser->type, args,
+		set_source_path(&path, args, parent_path), __func__,
 		"Expected OpenAPI type=%s%s%s (Slurm type=%s) but got OpenAPI type=%s%s%s (Slurm type=%s)",
 		oas_type, (oas_format ? " format=" : ""),
 		(oas_format ? oas_format : ""),
-		data_type_to_string(openapi_type_format_to_data_type(
-			parser->obj_openapi)),
-		found_type, (found_format ? " format=" : ""),
+		data_type_to_string(oas_data_type), found_type,
+		(found_format ? " format=" : ""),
 		(found_format ? found_format : ""), data_get_type_string(src));
 
 	xfree(path);
@@ -669,7 +904,7 @@ static void _parse_check_openapi(const parser_t *const parser, data_t *src,
 extern int parse(void *dst, ssize_t dst_bytes, const parser_t *const parser,
 		 data_t *src, args_t *args, data_t *parent_path)
 {
-	int rc;
+	int rc = SLURM_SUCCESS;
 	char *path = NULL;
 
 	check_parser(parser);
@@ -690,7 +925,8 @@ extern int parse(void *dst, ssize_t dst_bytes, const parser_t *const parser,
 		if (parser->required) {
 			if ((rc = on_error(PARSING, parser->type, args,
 					   ESLURM_DATA_PATH_NOT_FOUND,
-					   set_source_path(&path, parent_path),
+					   set_source_path(&path, args,
+							   parent_path),
 					   __func__,
 					   "Missing required field '%s' in dictionary",
 					   parser->key)))
@@ -698,7 +934,8 @@ extern int parse(void *dst, ssize_t dst_bytes, const parser_t *const parser,
 		} else {
 			/* field is missing but not required */
 			log_flag(DATA, "%s: skip parsing missing %s to %zd byte object %s(0x%" PRIxPTR "+%zd)%s%s via parser %s(0x%" PRIxPTR ")",
-				__func__, set_source_path(&path, parent_path),
+				__func__, set_source_path(&path, args,
+							  parent_path),
 				(dst_bytes == NO_VAL ? -1 : dst_bytes),
 				parser->obj_type_string, (uintptr_t) dst,
 				(parser->ptr_offset == NO_VAL ?
@@ -714,7 +951,7 @@ extern int parse(void *dst, ssize_t dst_bytes, const parser_t *const parser,
 	}
 
 	log_flag(DATA, "%s: BEGIN: parsing %s{%s(0x%" PRIxPTR ")} to %zd byte object %s(0x%" PRIxPTR "+%zd)%s%s via parser %s(0x%" PRIxPTR ")",
-		 __func__, set_source_path(&path, parent_path),
+		 __func__, set_source_path(&path, args, parent_path),
 		 data_get_type_string(src),
 		 (uintptr_t) src, (dst_bytes == NO_VAL ? -1 : dst_bytes),
 		 parser->obj_type_string, (uintptr_t) dst,
@@ -737,14 +974,41 @@ extern int parse(void *dst, ssize_t dst_bytes, const parser_t *const parser,
 		rc = _parse_list(parser, dst, src, args, parent_path);
 		break;
 	case PARSER_MODEL_ARRAY:
+	{
 		xassert(parser->fields);
 		verify_parser_not_sliced(parser);
 
-		/* recursively run the child parsers */
-		for (int i = 0; !rc && (i < parser->field_count); i++)
-			rc = _parser_linked(args, parser, &parser->fields[i],
-					    src, dst, parent_path);
+		if (data_get_type(src) != DATA_TYPE_DICT) {
+			rc = on_error(PARSING, parser->type, args,
+				      ESLURM_DATA_EXPECTED_DICT,
+				      set_source_path(&path, args, parent_path),
+				      __func__,
+				      "Rejecting %s when dictionary expected",
+				      data_get_type_string(src));
+		} else {
+			/* recursively run the child parsers */
+			for (int i = 0; !rc && (i < parser->field_count); i++)
+				rc = _parser_linked(args, parser,
+						    &parser->fields[i], src,
+						    dst, parent_path);
+
+			if (!is_fast_mode(args)) {
+				parse_marray_args_t aargs = {
+					.magic = MAGIC_FOREACH_PARSE_MARRAY,
+					.args = args,
+					.array = parser,
+					.parent_path = parent_path,
+				};
+
+				aargs.path = data_set_list(data_new());
+				(void) data_dict_for_each(src,
+							  _foreach_parse_marray,
+							  &aargs);
+				FREE_NULL_DATA(aargs.path);
+			}
+		}
 		break;
+	}
 	case PARSER_MODEL_PTR:
 		verify_parser_not_sliced(parser);
 		rc = _parse_pointer(parser, dst, src, args, parent_path);
@@ -758,9 +1022,13 @@ extern int parse(void *dst, ssize_t dst_bytes, const parser_t *const parser,
 	case PARSER_MODEL_COMPLEX:
 		xassert(parser->parse != _parse_list);
 		verify_parser_not_sliced(parser);
-		_parse_check_openapi(parser, src, args, parent_path);
+
+		if (!is_fast_mode(args) && !is_complex_mode(args))
+			_parse_check_openapi(parser, src, args, parent_path);
+
 		rc = parser->parse(parser, dst, src, args, parent_path);
 		break;
+	case PARSER_MODEL_ARRAY_LINKED_EXPLODED_FLAG_ARRAY_FIELD:
 	case PARSER_MODEL_ARRAY_LINKED_FIELD:
 		fatal_abort("%s: link model not allowed %u",
 			    __func__, parser->model);
@@ -777,7 +1045,7 @@ extern int parse(void *dst, ssize_t dst_bytes, const parser_t *const parser,
 
 cleanup:
 	log_flag(DATA, "%s: END: parsing %s{%s(0x%" PRIxPTR ")} to %zd byte object %s(0x%" PRIxPTR "+%zd)%s%s via parser %s(0x%" PRIxPTR ") rc[%d]:%s",
-		 __func__, set_source_path(&path, parent_path),
+		 __func__, set_source_path(&path, args, parent_path),
 		 data_get_type_string(src), (uintptr_t) src,
 		 (dst_bytes == NO_VAL ? -1 : dst_bytes),
 		 parser->obj_type_string, (uintptr_t) dst, (parser->ptr_offset
@@ -849,6 +1117,74 @@ static bool _match_flag_equal(const parser_t *const parser, void *src,
 	return found;
 }
 
+static void _dump_flag_bit_array_flag(args_t *args, void *src, data_t *dst,
+				      const parser_t *const parser,
+				      const flag_bit_t *bit, bool set_bool,
+				      uint64_t *used_equal_bits)
+{
+	bool found;
+
+	if (bit->type == FLAG_BIT_TYPE_BIT)
+		found = _match_flag_bit(parser, src, bit, *used_equal_bits);
+	else if (bit->type == FLAG_BIT_TYPE_EQUAL)
+		found = _match_flag_equal(parser, src, bit, used_equal_bits);
+	else
+		fatal_abort("%s: invalid bit_flag_t", __func__);
+
+	if (set_bool)
+		data_set_bool(dst, found);
+	else if (found) {
+		data_t *dst_flag;
+
+		if (parser->single_flag)
+			dst_flag = dst;
+		else
+			dst_flag = data_list_append(dst);
+
+		data_set_string(dst_flag, bit->name);
+	}
+
+	if (slurm_conf.debug_flags & DEBUG_FLAG_DATA) {
+		const char *type;
+		uint64_t value;
+
+		if (parser->size == sizeof(uint64_t)) {
+			uint64_t *flags = src;
+			value = *flags;
+		} else if (parser->size == sizeof(uint32_t)) {
+			uint32_t *flags = src;
+			value = *flags;
+		} else if (parser->size == sizeof(uint16_t)) {
+			uint16_t *flags = src;
+			value = *flags;
+		} else if (parser->size == sizeof(uint8_t)) {
+			uint8_t *flags = src;
+			value = *flags;
+		} else {
+			fatal_abort("invalid parser flag size: %zu",
+				    parser->size);
+		}
+
+		if (bit->type == FLAG_BIT_TYPE_BIT)
+			type = "bit";
+		else if (bit->type == FLAG_BIT_TYPE_EQUAL)
+			type = "bit-equals";
+		else
+			type = "INVALID";
+
+		log_flag(DATA, "%s: %s \"%s\" flag %s %s(%s[0x%"PRIx64"] & %s[0x%"PRIx64"]) & 0x%"PRIx64" = 0x%"PRIx64" for %zd byte %s(0x%" PRIxPTR "+%zd)->%s with parser %s(0x%" PRIxPTR ") to data %s[0x%" PRIxPTR "]",
+			 __func__, (found ? "appending matched" : "skipping"),
+			 bit->name, type, bit->name, bit->mask_name, bit->mask,
+			 bit->flag_name, bit->value, value,
+			 (bit->mask & value & bit->value), parser->size,
+			 parser->obj_type_string, (uintptr_t) src,
+			 parser->ptr_offset, parser->field_name,
+			 parser->type_string, (uintptr_t) parser,
+			 data_get_type_string(dst),
+			 (uintptr_t) dst);
+	}
+}
+
 static int _dump_flag_bit_array(args_t *args, void *src, data_t *dst,
 				const parser_t *const parser)
 {
@@ -858,68 +1194,18 @@ static int _dump_flag_bit_array(args_t *args, void *src, data_t *dst,
 	xassert(args->magic == MAGIC_ARGS);
 	check_parser(parser);
 
-	if (data_get_type(dst) == DATA_TYPE_NULL)
-		data_set_list(dst);
-	if (data_get_type(dst) != DATA_TYPE_LIST)
-		return ESLURM_DATA_CONV_FAILED;
-
-	for (int8_t i = 0; !rc && (i < parser->flag_bit_array_count); i++) {
-		bool found;
-		const flag_bit_t *bit = &parser->flag_bit_array[i];
-
-		if (bit->type == FLAG_BIT_TYPE_BIT)
-			found = _match_flag_bit(parser, src, bit,
-						used_equal_bits);
-		else if (bit->type == FLAG_BIT_TYPE_EQUAL)
-			found = _match_flag_equal(parser, src, bit,
-						  &used_equal_bits);
-		else
-			fatal_abort("%s: invalid bit_flag_t", __func__);
-
-		if (found)
-			data_set_string(data_list_append(dst), bit->name);
-
-		if (slurm_conf.debug_flags & DEBUG_FLAG_DATA) {
-			const char *type;
-			uint64_t value;
-
-			if (parser->size == sizeof(uint64_t)) {
-				uint64_t *flags = src;
-				value = *flags;
-			} else if (parser->size == sizeof(uint32_t)) {
-				uint32_t *flags = src;
-				value = *flags;
-			} else if (parser->size == sizeof(uint16_t)) {
-				uint16_t *flags = src;
-				value = *flags;
-			} else if (parser->size == sizeof(uint8_t)) {
-				uint8_t *flags = src;
-				value = *flags;
-			} else {
-				fatal_abort("invalid parser flag size: %zu",
-					    parser->size);
-			}
-
-			if (bit->type == FLAG_BIT_TYPE_BIT)
-				type = "bit";
-			else if (bit->type == FLAG_BIT_TYPE_EQUAL)
-				type = "bit-equals";
-			else
-				type = "INVALID";
-
-			log_flag(DATA, "%s: %s \"%s\" flag %s %s(%s[0x%"PRIx64"] & %s[0x%"PRIx64"]) & 0x%"PRIx64" = 0x%"PRIx64" for %zd byte %s(0x%" PRIxPTR "+%zd)->%s with parser %s(0x%" PRIxPTR ") to data %s[0x%" PRIxPTR "]",
-				 __func__, (found ? "appending matched" : "skipping"),
-				 bit->name, type, bit->name, bit->mask_name,
-				 bit->mask, bit->flag_name, bit->value, value,
-				 (bit->mask & value & bit->value),
-				 parser->size, parser->obj_type_string,
-				 (uintptr_t) src, parser->ptr_offset,
-				 parser->field_name, parser->type_string,
-				 (uintptr_t) parser, data_get_type_string(dst),
-				 (uintptr_t) dst);
-		}
+	if (!parser->single_flag) {
+		if (data_get_type(dst) == DATA_TYPE_NULL)
+			data_set_list(dst);
+		if (data_get_type(dst) != DATA_TYPE_LIST)
+			return ESLURM_DATA_CONV_FAILED;
 	}
 
+	for (int8_t i = 0; !rc && (i < parser->flag_bit_array_count); i++)
+		_dump_flag_bit_array_flag(args, src, dst, parser,
+					  &parser->flag_bit_array[i], false,
+					  &used_equal_bits);
+
 	return SLURM_SUCCESS;
 }
 
@@ -934,8 +1220,8 @@ static int _foreach_dump_list(void *obj, void *arg)
 	xassert(args->parser->ptr_offset == NO_VAL);
 
 	/* we don't know the size of the items in the list */
-	if (data_parser_p_dump(args->args, args->parser->list_type, obj, NO_VAL,
-			       item))
+	if (dump(&obj, NO_VAL, find_parser_by_type(args->parser->list_type),
+		 item, args->args))
 		return -1;
 
 	return 0;
@@ -979,11 +1265,17 @@ static int _dump_list(const parser_t *const parser, void *src, data_t *dst,
 static int _dump_pointer(const parser_t *const parser, void *src, data_t *dst,
 			 args_t *args)
 {
-	const parser_t *const pt = find_parser_by_type(parser->pointer_type);
+	const parser_t *pt = find_parser_by_type(parser->pointer_type);
 	void **ptr = src;
 
-	if (!*ptr) {
-		if ((pt->model == PARSER_MODEL_ARRAY) ||
+	if (!*ptr && !is_complex_mode(args)) {
+		/* Fully resolve pointer on NULL to use correct model */
+		while (pt->pointer_type)
+			pt = find_parser_by_type(pt->pointer_type);
+
+		if (parser->allow_null_pointer) {
+			xassert(data_get_type(dst) == DATA_TYPE_NULL);
+		} else if ((pt->model == PARSER_MODEL_ARRAY) ||
 		    (pt->obj_openapi == OPENAPI_FORMAT_OBJECT)) {
 			/*
 			 * OpenAPI clients can't handle a null instead of an
@@ -1024,9 +1316,9 @@ static int _dump_nt_array(const parser_t *const parser, void *src, data_t *dst,
 			return SLURM_SUCCESS;
 
 		for (int i = 0; !rc && array[i]; i++) {
-			rc = data_parser_p_dump(args, parser->array_type,
-						array[i], NO_VAL,
-						data_list_append(dst));
+			rc = dump(array[i], NO_VAL,
+				  find_parser_by_type(parser->array_type),
+				  data_list_append(dst), args);
 		}
 	} else if (parser->model == PARSER_MODEL_NT_ARRAY) {
 		const parser_t *const ap =
@@ -1048,8 +1340,9 @@ static int _dump_nt_array(const parser_t *const parser, void *src, data_t *dst,
 			if (done)
 				break;
 
-			rc = data_parser_p_dump(args, parser->array_type, ptr,
-						NO_VAL, data_list_append(dst));
+			rc = dump(ptr, NO_VAL,
+				  find_parser_by_type(parser->array_type),
+				  data_list_append(dst), args);
 		}
 	} else {
 		fatal_abort("invalid model");
@@ -1061,12 +1354,12 @@ static int _dump_nt_array(const parser_t *const parser, void *src, data_t *dst,
 static int _dump_linked(args_t *args, const parser_t *const array,
 			const parser_t *const parser, void *src, data_t *dst)
 {
-	int rc;
+	int rc = SLURM_SUCCESS;
 
 	check_parser(parser);
 	verify_parser_sliced(parser);
 
-	if (parser->ptr_offset != NO_VAL)
+	if ((parser->ptr_offset != NO_VAL) && src)
 		src += parser->ptr_offset;
 
 	/*
@@ -1091,7 +1384,6 @@ static int _dump_linked(args_t *args, const parser_t *const array,
 			 (uintptr_t) src, array->field_name,
 			 array->ptr_offset, (uintptr_t) dst,
 			 array->key, (uintptr_t) dst);
-		rc = SLURM_SUCCESS;
 		goto cleanup;
 	}
 
@@ -1145,7 +1437,25 @@ static int _dump_linked(args_t *args, const parser_t *const array,
 				fatal_abort("invalid type");
 		};
 
-		rc = SLURM_SUCCESS;
+		return SLURM_SUCCESS;
+	}
+
+	if (parser->model ==
+	    PARSER_MODEL_ARRAY_LINKED_EXPLODED_FLAG_ARRAY_FIELD) {
+		uint64_t used_equal_bits = 0;
+
+		if (data_get_type(dst) == DATA_TYPE_NULL)
+			data_set_dict(dst);
+
+		for (int i = 0; i < parser->flag_bit_array_count; i++) {
+			const flag_bit_t *bit = &parser->flag_bit_array[i];
+			data_t *bit_dst = data_define_dict_path(dst, bit->name);
+
+			xassert(src);
+			_dump_flag_bit_array_flag(args, src, bit_dst, parser,
+						  bit, true, &used_equal_bits);
+		}
+
 		goto cleanup;
 	}
 
@@ -1171,6 +1481,21 @@ cleanup:
 	return rc;
 }
 
+static void _check_dump(const parser_t *const parser, data_t *dst, args_t *args)
+{
+	/*
+	 * Resultant dump must be the proscribed OpenAPI compatible data_t type.
+	 * Anything else will break most generated OpenAPI clients.
+	 */
+	if (parser->obj_openapi == OPENAPI_FORMAT_INVALID)
+		return;
+
+	if (!is_complex_mode(args)) {
+		xassert(data_get_type(dst) ==
+			openapi_type_format_to_data_type(parser->obj_openapi));
+	}
+}
+
 extern int dump(void *src, ssize_t src_bytes, const parser_t *const parser,
 		data_t *dst, args_t *args)
 {
@@ -1186,13 +1511,18 @@ extern int dump(void *src, ssize_t src_bytes, const parser_t *const parser,
 	xassert(dst && (data_get_type(dst) != DATA_TYPE_NONE));
 	xassert(args->magic == MAGIC_ARGS);
 	xassert((src_bytes == NO_VAL) || (src_bytes > 0));
-	xassert(src);
+
 	/*
 	 * Make sure the source object is the same size since there is no
 	 * way to dump value of __typeof__ as a value in C.
 	 */
 	xassert((src_bytes == NO_VAL) || (src_bytes == parser->size));
 
+	if (args->flags & FLAG_SPEC_ONLY) {
+		set_openapi_schema(dst, parser, args);
+		return SLURM_SUCCESS;
+	}
+
 	if ((rc = load_prereqs(DUMPING, parser, args)))
 		goto done;
 
@@ -1258,7 +1588,9 @@ extern int dump(void *src, ssize_t src_bytes, const parser_t *const parser,
 		 */
 
 		rc = parser->dump(parser, src, dst, args);
+		_check_dump(parser, dst, args);
 		break;
+	case PARSER_MODEL_ARRAY_LINKED_EXPLODED_FLAG_ARRAY_FIELD:
 	case PARSER_MODEL_ARRAY_LINKED_FIELD:
 		fatal_abort("%s: link model not allowed %u",
 			    __func__, parser->model);
