@@ -1,7 +1,7 @@
 /*****************************************************************************\
  *  gpu_rsmi.c - Support rsmi interface to an AMD GPU.
  *****************************************************************************
- *  Copyright (C) 2019 SchedMD LLC
+ *  Copyright (C) SchedMD LLC.
  *  Copyright (c) 2019, Advanced Micro Devices, Inc. All rights reserved.
  *  Written by Advanced Micro Devices,
  *  who borrowed heavily from SLURM gpu and nvml plugin.
@@ -58,6 +58,9 @@ static bitstr_t	*saved_gpus;
  */
 #define RSMI_STRING_BUFFER_SIZE			80
 
+/* ROCM release version >= 6.0.0 required for gathering usage */
+#define RSMI_REQ_VERSION_USAGE 6
+
 /*
  * PCI information about a GPU device.
  */
@@ -114,12 +117,25 @@ const uint32_t	plugin_version		= SLURM_VERSION_NUMBER;
 static int gpumem_pos = -1;
 static int gpuutil_pos = -1;
 
+static bool get_usage = true;
+
+static void _rsmi_get_version(char *version, unsigned int len);
+
 extern int init(void)
 {
 	rsmi_init(0);
 
 	if (running_in_slurmstepd()) {
-		gpu_get_tres_pos(&gpumem_pos, &gpuutil_pos);
+		char version[RSMI_STRING_BUFFER_SIZE];
+
+		_rsmi_get_version(version, RSMI_STRING_BUFFER_SIZE);
+		/*
+		 * If version < RSMI_REQ_VERSION_USAGE get_usage will be set to
+		 * false, so we won't set gpumem_pos and gpuutil_pos which
+		 * effectively disables gpu accounting.
+		 */
+		if (get_usage)
+			gpu_get_tres_pos(&gpumem_pos, &gpuutil_pos);
 	}
 
 	debug("%s: %s loaded", __func__, plugin_name);
@@ -233,7 +249,7 @@ static void _rsmi_print_freqs(uint32_t dv_ind, log_level_t l)
 		return;
 
 	qsort(mem_freqs, size, sizeof(unsigned int),
-	      gpu_common_sort_freq_descending);
+	      slurm_sort_uint_list_desc);
 	if ((size > 1) && (mem_freqs[0] <= mem_freqs[(size)-1])) {
 		error("%s: memory frequencies are not stored in descending order!",
 		      __func__);
@@ -247,7 +263,7 @@ static void _rsmi_print_freqs(uint32_t dv_ind, log_level_t l)
 		return;
 
 	qsort(gfx_freqs, size, sizeof(unsigned int),
-	      gpu_common_sort_freq_descending);
+	      slurm_sort_uint_list_desc);
 	if ((size > 1) && (gfx_freqs[0] <= gfx_freqs[(size)-1])) {
 		error("%s: Graphics frequencies are not stored in descending order!",
 		      __func__);
@@ -287,7 +303,7 @@ static void _rsmi_get_nearest_freqs(uint32_t dv_ind,
 
 	memcpy(mem_freqs_sort, mem_freqs, mem_freqs_size*sizeof(unsigned int));
 	qsort(mem_freqs_sort, mem_freqs_size, sizeof(unsigned int),
-	      gpu_common_sort_freq_descending);
+	      slurm_sort_uint_list_desc);
 	if ((mem_freqs_size > 1) &&
 	    (mem_freqs_sort[0] <= mem_freqs_sort[(mem_freqs_size)-1])) {
 		error("%s: memory frequencies are not stored in descending order!",
@@ -311,7 +327,7 @@ static void _rsmi_get_nearest_freqs(uint32_t dv_ind,
 
 	memcpy(gfx_freqs_sort, gfx_freqs, gfx_freqs_size*sizeof(unsigned int));
 	qsort(gfx_freqs_sort, gfx_freqs_size, sizeof(unsigned int),
-	      gpu_common_sort_freq_descending);
+	      slurm_sort_uint_list_desc);
 	if ((gfx_freqs_size > 1) &&
 	    (gfx_freqs_sort[0] <= gfx_freqs_sort[(gfx_freqs_size)-1])) {
 		error("%s: graphics frequencies are not stored in descending order!",
@@ -650,8 +666,14 @@ static void _rsmi_get_version(char *version, unsigned int len)
 		error("RSMI: Failed to get the version error: %s",
 		      status_string);
 		version[0] = '\0';
-	} else
-		sprintf(version, "%s", rsmi_version.build);
+	} else {
+		snprintf(version, len, "%s", rsmi_version.build);
+		if (rsmi_version.major < RSMI_REQ_VERSION_USAGE) {
+			get_usage = false;
+			error("%s: GPU usage accounting disabled. RSMI version >= 6.0.0 required.",
+			      __func__);
+		}
+	}
 }
 
 /*
@@ -928,11 +950,6 @@ static List _get_system_gpu_list_rsmi(node_config_load_t *node_config)
 	return gres_list_system;
 }
 
-extern int gpu_p_reconfig(void)
-{
-	return SLURM_SUCCESS;
-}
-
 extern List gpu_p_get_system_gpu_list(node_config_load_t *node_config)
 {
 	List gres_list_system = _get_system_gpu_list_rsmi(node_config);
