@@ -40,6 +40,8 @@
  *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA.
  \*****************************************************************************/
 
+#define _GNU_SOURCE
+
 #include <pthread.h>
 #include <sched.h>
 #include <poll.h>
@@ -57,6 +59,8 @@
 
 static pthread_mutex_t agent_mutex = PTHREAD_MUTEX_INITIALIZER;
 static pthread_cond_t agent_running_cond = PTHREAD_COND_INITIALIZER;
+static pthread_mutex_t abort_mutex = PTHREAD_MUTEX_INITIALIZER;
+static pthread_cond_t abort_mutex_cond = PTHREAD_COND_INITIALIZER;
 
 static eio_handle_t *_io_handle = NULL;
 static eio_handle_t *_abort_handle = NULL;
@@ -127,7 +131,8 @@ static int _server_conn_read(eio_obj_t *obj, List objs)
 			return 0;
 		}
 
-		while ((fd = accept(obj->fd, &addr, &size)) < 0) {
+		while ((fd = accept4(obj->fd, &addr, &size,
+				     (SOCK_CLOEXEC | SOCK_NONBLOCK))) < 0) {
 			if (errno == EINTR)
 				continue;
 			if (errno == EAGAIN) /* No more connections */
@@ -224,9 +229,7 @@ static void _shutdown_timeout_fds(void);
 
 #define SETUP_FDS(fds) { \
 	fd_set_nonblocking(fds[0]);	\
-	fd_set_close_on_exec(fds[0]);	\
 	fd_set_nonblocking(fds[1]);	\
-	fd_set_close_on_exec(fds[1]);	\
 	}
 
 static int _setup_timeout_fds(void)
@@ -236,14 +239,14 @@ static int _setup_timeout_fds(void)
 	timer_data.work_in = timer_data.work_out = -1;
 	timer_data.stop_in = timer_data.stop_out = -1;
 
-	if (pipe(fds)) {
+	if (pipe2(fds, O_CLOEXEC)) {
 		return SLURM_ERROR;
 	}
 	SETUP_FDS(fds);
 	timer_data.work_in = fds[0];
 	timer_data.work_out = fds[1];
 
-	if (pipe(fds)) {
+	if (pipe2(fds, O_CLOEXEC)) {
 		_shutdown_timeout_fds();
 		return SLURM_ERROR;
 	}
@@ -350,6 +353,11 @@ rwfail:
 static void *_pmix_abort_thread(void *args)
 {
 	PMIXP_DEBUG("Start abort thread");
+
+	slurm_mutex_lock(&abort_mutex);
+	slurm_cond_signal(&abort_mutex_cond);
+	slurm_mutex_unlock(&abort_mutex);
+
 	eio_handle_mainloop(_abort_handle);
 	PMIXP_DEBUG("Abort thread exit");
 	return NULL;
@@ -360,8 +368,14 @@ int pmixp_abort_agent_start(char ***env)
 	int abort_server_socket = -1;
 	slurm_addr_t abort_server;
 	eio_obj_t *obj;
+	uint16_t *ports;
 
-	if ((abort_server_socket = slurm_init_msg_engine_port(0)) < 0) {
+	slurm_mutex_lock(&abort_mutex);
+	if ((ports = slurm_get_srun_port_range()))
+		abort_server_socket = slurm_init_msg_engine_ports(ports);
+	else
+		abort_server_socket = slurm_init_msg_engine_port(0);
+	if (abort_server_socket < 0) {
 		PMIXP_ERROR("slurm_init_msg_engine_port() failed: %m");
 		return SLURM_ERROR;
 	}
@@ -382,16 +396,22 @@ int pmixp_abort_agent_start(char ***env)
 	eio_new_initial_obj(_abort_handle, obj);
 	slurm_thread_create(&_abort_tid, _pmix_abort_thread, NULL);
 
+	/* wait for the abort EIO thread to initialize */
+	slurm_cond_wait(&abort_mutex_cond, &abort_mutex);
+
+	slurm_mutex_unlock(&abort_mutex);
 	return SLURM_SUCCESS;
 }
 
 int pmixp_abort_agent_stop(void)
 {
+	slurm_mutex_lock(&abort_mutex);
 	if (_abort_tid) {
 		eio_signal_shutdown(_abort_handle);
-		pthread_join(_abort_tid, NULL);
-		_abort_tid = 0;
+		slurm_thread_join(_abort_tid);
 	}
+	slurm_mutex_unlock(&abort_mutex);
+
 	return pmixp_abort_code_get();
 }
 
@@ -452,16 +472,14 @@ int pmixp_agent_stop(void)
 	if (_agent_tid) {
 		eio_signal_shutdown(_io_handle);
 		/* wait for the agent thread to stop */
-		pthread_join(_agent_tid, NULL);
-		_agent_tid = 0;
+		slurm_thread_join(_agent_tid);
 	}
 
 	if (_timer_tid) {
 		/* cancel timer */
 		if (write(timer_data.stop_out, &c, 1) == -1)
 			rc = SLURM_ERROR;
-		pthread_join(_timer_tid, NULL);
-		_timer_tid = 0;
+		slurm_thread_join(_timer_tid);
 
 		/* close timer fds */
 		_shutdown_timeout_fds();
