@@ -48,9 +48,6 @@ static bool running_flag = false;
 static pthread_t tid;
 static uint16_t timeout;
 static char *program_name;
-#ifdef HAVE_NATIVE_CRAY
-static uint32_t recorded_het_jobid = NO_VAL;
-#endif
 static uint32_t recorded_jobid = NO_VAL;
 static uint32_t recorded_stepid = NO_VAL;
 
@@ -76,9 +73,6 @@ void step_terminate_monitor_start(stepd_step_rec_t *step)
 	running_flag = true;
 	slurm_thread_create(&tid, _monitor, step);
 
-#ifdef HAVE_NATIVE_CRAY
-	recorded_het_jobid = step->het_job_id;
-#endif
 	recorded_jobid = step->step_id.job_id;
 	recorded_stepid = step->step_id.step_id;
 
@@ -100,8 +94,7 @@ void step_terminate_monitor_stop(void)
 	slurm_cond_signal(&cond);
 	slurm_mutex_unlock(&lock);
 
-	if (pthread_join(tid, NULL) != 0)
-		error("%s pthread_join: %m", __func__);
+	slurm_thread_join(tid);
 
 	xfree(program_name);
 }
@@ -216,7 +209,6 @@ static int _call_external_program(stepd_step_rec_t *step)
 		/* child */
 		char *argv[2];
 		char **env = NULL;
-		uint32_t jobid;
 
 		/* container_g_join needs to be called in the
 		   forked process part of the fork to avoid a race
@@ -224,16 +216,8 @@ static int _call_external_program(stepd_step_rec_t *step)
 		   detacts itself from a child before we add the pid
 		   to the container in the parent of the fork.
 		*/
-#ifdef HAVE_NATIVE_CRAY
-		if (recorded_het_jobid && (recorded_het_jobid != NO_VAL))
-			jobid = recorded_het_jobid;
-		else
-			jobid = recorded_jobid;
-#else
-		jobid = recorded_jobid;
-#endif
-		if (container_g_join(jobid, getuid()) != SLURM_SUCCESS)
-			error("container_g_join(%u): %m", jobid);
+		if (container_g_join(recorded_jobid, getuid()) != SLURM_SUCCESS)
+			error("container_g_join(%u): %m", recorded_jobid);
 		env = env_array_create();
 		env_array_append_fmt(&env, "SLURM_JOBID", "%u", recorded_jobid);
 		env_array_append_fmt(&env, "SLURM_JOB_ID", "%u", recorded_jobid);
